@@ -1339,8 +1339,9 @@ public class Wdic2Xml {
         Element layerElem = _appendLayer(menu, id);
         Element refElem = _appendIdReference(layerElem, "MENU:group");
         _appendRawText(refElem, "グループ");
+        _appendRawText(layerElem, " > ");
         refElem = _appendIdReference(layerElem, "MENU:group:" + grpId);
-        _appendRawText(refElem, " > " + wdic.getGroupName());
+        _appendRawText(refElem, wdic.getGroupName());
         _appendRawText(layerElem, " > " + wdic.getPartName() + "編");
         _appendNewLine(layerElem);
         List<WdicItem> items = wdic.getWdicItems();
@@ -1518,6 +1519,8 @@ public class Wdic2Xml {
      */
     private void _appendRawText(Node node, String str) {
         if (str != null && str.trim().length() > 0) {
+            str = str.replace((char)0x3099, (char)0x309b);
+            str = str.replace((char)0x309a, (char)0x309c);
             Text text = node.getOwnerDocument().createTextNode(str);
             node.appendChild(text);
             _checkCharacter(text);
@@ -1652,6 +1655,9 @@ public class Wdic2Xml {
      * @param linkBlock リンク部の場合はtrue
      */
     private void _appendText(WdicItem item, Node node, String str, boolean linkBlock) {
+        String grpId = item.getWdic().getGroupId();
+        String partId = item.getWdic().getPartId();
+        String itemId = grpId + ":" + partId + ":" + item.getHead();
         StringBuilder buf = new StringBuilder();
         int len = str.length();
         for (int i=0; i<len; i++) {
@@ -1738,7 +1744,7 @@ public class Wdic2Xml {
                                 gid = ref.substring(2, idx3);
                                 file = ref.substring(idx3+1);
                             } else {
-                                gid = item.getWdic().getGroup().getGroupId();
+                                gid = grpId;
                                 file = ref.substring(2);
                             }
                             Element refElem = null;
@@ -1787,7 +1793,7 @@ public class Wdic2Xml {
                             }
                             if (StringUtils.isBlank(gid)) {
                                 // 同一グループ内
-                                gid = item.getWdic().getGroup().getGroupId();
+                                gid = grpId;
                             }
                             WdicGroup group = _groupList.getGroup(gid);
                             if (group != null) {
@@ -2044,7 +2050,7 @@ public class Wdic2Xml {
             } else {
                 if (!"unit".equals(name)
                     && !"date".equals(name) && !"dt".equals(name)) {
-                    _log.error("unknown function name: " + name);
+                    _log.error("unknown function name: " + itemId + " [" + name + "]");
                 }
                 _appendRawText(node, buf.toString());
                 buf.delete(0, buf.length());
