@@ -728,6 +728,8 @@ public class WdicTableItem {
      * @return 描画後x座標
      */
     private int _drawRawText(Graphics2D g2, String str, int x, int y) {
+        str = str.replace((char)0x3099, (char)0x309b);
+        str = str.replace((char)0x309a, (char)0x309c);
         Font origFont = g2.getFont();
         while (StringUtils.isNotEmpty(str)) {
             int codePoint = str.codePointAt(0);
