@@ -6,11 +6,12 @@ import java.util.List;
 /**
  * テーブル行クラス。
  *
+ * @author Hisaya FUKUMOTO
  */
 public class WdicTableRow {
 
     /** 要素リスト */
-    ArrayList<WdicTableItem> _items = new ArrayList<WdicTableItem>();
+    List<WdicTableItem> _items = new ArrayList<WdicTableItem>();
 
 
     /**
