@@ -1,12 +1,13 @@
 package fuku.xml2eb.converter.zipcode;
 
+import java.awt.image.BufferedImage;
 import java.io.File;
 import java.io.IOException;
 import java.text.SimpleDateFormat;
-import java.util.Date;
 import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
+import java.util.TreeMap;
 import javax.xml.parsers.DocumentBuilderFactory;
 import javax.xml.parsers.DocumentBuilder;
 import javax.xml.parsers.ParserConfigurationException;
@@ -17,12 +18,13 @@ import org.w3c.dom.Node;
 import org.w3c.dom.NodeList;
 import org.w3c.dom.Text;
 
+import org.apache.commons.io.FilenameUtils;
 import org.apache.commons.logging.LogFactory;
 import org.apache.commons.logging.Log;
 
 import fuku.eb4j.util.HexUtil;
 import fuku.xml2eb.CatalogInfo;
-import fuku.xml2eb.util.UnicodeUtil;
+import fuku.xml2eb.util.FontUtil;
 import fuku.xml2eb.util.WordUtil;
 import fuku.xml2eb.util.XmlUtil;
 
@@ -36,6 +38,7 @@ public class ZipCode2Xml {
     /** プロブラム名 */
     private static final String _PROGRAM = ZipCode2Xml.class.getName();
 
+    private static final String GAIJI_DIR = "gaiji";
     private static final String BOOK_XML = "book.xml";
 
     private static final String BOOK_TITLE1 = "郵便番号";
@@ -46,13 +49,19 @@ public class ZipCode2Xml {
         "0x" + HexUtil.toHexString(CatalogInfo.TYPE_GENERAL, 2);
 
     /** ログ */
-    private transient Log _log = null;
+    private Log _log = null;
     /** ベースディレクトリ */
     private File _basedir = null;
     /** 全国一括郵便番号 */
     private ZipCodeKen _ken = null;
     /** 事業所個別郵便番号 */
     private ZipCodeJigyosyo _jigyosyo = null;
+    /** 外字マップ */
+    private Map<String,String> _gaijiMap = null;
+    /** 全国一括郵便番号用外字マップ */
+    private Map<String,String> _kenGaijiMap = null;
+    /** 事業所個別郵便番号用外字マップ */
+    private Map<String,String> _jigyosyoGaijiMap = null;
 
 
     /**
@@ -101,6 +110,8 @@ public class ZipCode2Xml {
         _ken = new ZipCodeKen(file);
         file = new File(_basedir, "JIGYOSYO.CSV");
         _jigyosyo = new ZipCodeJigyosyo(file);
+        _kenGaijiMap = new TreeMap<String,String>();
+        _jigyosyoGaijiMap = new TreeMap<String,String>();
 
         DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
         DocumentBuilder builder = factory.newDocumentBuilder();
@@ -109,6 +120,7 @@ public class ZipCode2Xml {
         doc.appendChild(book);
 
         _log.info("create zipcode subbook...");
+        _gaijiMap = _kenGaijiMap;
         Element subbook = _appendElement(book, "subbook");
         subbook.setAttribute("title", BOOK_TITLE1);
         subbook.setAttribute("dir", BOOK_DIR1);
@@ -117,8 +129,10 @@ public class ZipCode2Xml {
         _makeKenItemNode(content);
         _makeKenMenuNode(content);
         _makeKenCopyrightNode(content);
+        _makeFontNode(subbook);
 
         _log.info("create jigyosyo subbook...");
+        _gaijiMap = _jigyosyoGaijiMap;
         subbook = _appendElement(book, "subbook");
         subbook.setAttribute("title", BOOK_TITLE2);
         subbook.setAttribute("dir", BOOK_DIR2);
@@ -127,6 +141,7 @@ public class ZipCode2Xml {
         _makeJigyosyoItemNode(content);
         _makeJigyosyoMenuNode(content);
         _makeJigyosyoCopyrightNode(content);
+        _makeFontNode(subbook);
 
         file = new File(_basedir, BOOK_XML);
         _log.info("write file: " + file.getPath());
@@ -458,6 +473,36 @@ public class ZipCode2Xml {
         }
     }
 
+    /**
+     * 外字データノードを作成します。
+     *
+     * @param subbook subbookノード
+     */
+    private void _makeFontNode(Element subbook) {
+        if (_gaijiMap.isEmpty()) {
+            return;
+        }
+        Element font = _appendElement(subbook, "font");
+        File gaiji = new File(_basedir, GAIJI_DIR);
+        Iterator<Map.Entry<String,String>> it = _gaijiMap.entrySet().iterator();
+        while (it.hasNext()) {
+            Map.Entry<String,String> entry = it.next();
+            String name = entry.getKey();
+            String type = entry.getValue();
+            File file = new File(gaiji, name + ".xbm");
+            if (!file.exists()) {
+                _log.error("file not found: " + file.getPath());
+            }
+            String path = FilenameUtils.concat(GAIJI_DIR, file.getName());
+            Element charElem = _appendElement(font, "char");
+            charElem.setAttribute("name", name);
+            charElem.setAttribute("type", type);
+            Element dataElem = _appendElement(charElem, "data");
+            dataElem.setAttribute("size", "16");
+            dataElem.setAttribute("src", path);
+        }
+    }
+
     /**
      * テキストノードを追加します。
      *
@@ -468,6 +513,7 @@ public class ZipCode2Xml {
         if (str != null && str.trim().length() > 0) {
             Text text = node.getOwnerDocument().createTextNode(str);
             node.appendChild(text);
+            _checkCharacter(text);
         }
     }
 
@@ -532,6 +578,84 @@ public class ZipCode2Xml {
         elem.setAttribute("id", id);
         return elem;
     }
+
+    /**
+     * 使用されている文字が有効かどうかを確認します。
+     *
+     * @param node ノード
+     */
+    private void _checkCharacter(Node node) {
+        if (node.getNodeType() == Node.TEXT_NODE) {
+            Text text = (Text)node;
+            String str = text.getNodeValue();
+            int len = str.length();
+            int idx = 0;
+            while (idx<len) {
+                int codePoint = str.codePointAt(idx);
+                int cnt = Character.charCount(codePoint);
+                if (WordUtil.isValidChar(codePoint)) {
+                    idx += cnt;
+                    continue;
+                }
+                Character.UnicodeBlock unicodeBlock = Character.UnicodeBlock.of(codePoint);
+                String hex = HexUtil.toHexString(codePoint, 6);
+                String name = "U" + hex;
+                File dir = new File(_basedir, GAIJI_DIR);
+                if (!dir.exists()) {
+                    dir.mkdirs();
+                }
+                File file = new File(dir, name + ".xbm");
+                if (!file.exists()) {
+                    String s = String.valueOf(Character.toChars(codePoint));
+                    _log.info("unsupported character:"
+                              + " [U+" + hex + "]"
+                              + " '" + s + "'"
+                              + " " + unicodeBlock.toString());
+                    BufferedImage img = ZipCodeUtil.toImage(codePoint);
+                    try {
+                        FontUtil.writeXbm(img, file);
+                    } catch (IOException e) {
+                        _log.error(e.getMessage(), e);
+                        file.delete();
+                    } finally {
+                        if (img != null) {
+                            img.flush();
+                        }
+                    }
+                }
+                String type = _gaijiMap.get(name);
+                if (type == null) {
+                    type = FontUtil.getFontType(codePoint);
+                    _gaijiMap.put(name, type);
+                }
+
+                Node parent = text.getParentNode();
+                text = text.splitText(idx);
+                text.deleteData(0, cnt);
+                Element elem = text.getOwnerDocument().createElement("char");
+                elem.setAttribute("name", name);
+                elem.setAttribute("type", type);
+                parent.insertBefore(elem, text);
+
+                str = text.getNodeValue();
+                len = str.length();
+                idx = 0;
+            }
+        }
+        if (node.hasChildNodes()) {
+            NodeList nlist = node.getChildNodes();
+            int len = nlist.getLength();
+            for (int i=0; i<len; i++) {
+                Node child = nlist.item(i);
+                _checkCharacter(child);
+                int n = nlist.getLength();
+                if (len < n) {
+                    i += n - len;
+                    len = n;
+                }
+            }
+        }
+    }
 }
 
 // end of ZipCode2Xml.java
