@@ -987,6 +987,9 @@ public class SubBook {
                 type = KANJI;
             }
         }
+        if (_wordStyle[type] == null) {
+            return new NullSearcher();
+        }
         SingleWordSearcher searcher =
             new SingleWordSearcher(this, _wordStyle[type], SingleWordSearcher.EXACTWORD);
         searcher.search(b);
@@ -1016,6 +1019,9 @@ public class SubBook {
                 type = KANJI;
             }
         }
+        if (_wordStyle[type] == null) {
+            return new NullSearcher();
+        }
         SingleWordSearcher searcher =
             new SingleWordSearcher(this, _wordStyle[type], SingleWordSearcher.WORD);
         searcher.search(b);
@@ -1045,6 +1051,9 @@ public class SubBook {
                 type = KANJI;
             }
         }
+        if (_endwordStyle[type] == null) {
+            return new NullSearcher();
+        }
         SingleWordSearcher searcher =
             new SingleWordSearcher(this, _endwordStyle[type], SingleWordSearcher.ENDWORD);
         searcher.search(b);
