@@ -343,7 +343,6 @@ public class ImageUtil {
                 for (int y=height-1; y>=0; y--) {
                     image[idx++] = 0x00; // filter type
                     System.arraycopy(dib, off+lineBytes*y, line, 0, lineBytes);
-                    int shift = 0;
                     for (int x=0; x<width; x+=2) {
                         int palette = paletteOff + ((line[x/2] >>> 4) & 0x0f) * paletteSize;
                         image[idx++] = dib[palette+2]; // R
@@ -386,10 +385,9 @@ public class ImageUtil {
                         for (int i=0; i<3; i++) {
                             int pxl = (val & bitField[i]) >>> bitShift[i];
                             if (bitNum[i] != 255) {
-                                image[idx++] = (byte)(Math.round(pxl*255/bitNum[i]));
-                            } else {
-                                image[idx++] = (byte)pxl;
+                                pxl = Math.round(pxl*255.0f/bitNum[i]);
                             }
+                            image[idx++] = (byte)pxl;
                         }
                         // alpha
                         if (bitField[3] == 0) {
@@ -397,10 +395,9 @@ public class ImageUtil {
                         } else {
                             int pxl = (val & bitField[3]) >>> bitShift[3];
                             if (bitNum[3] != 255) {
-                                image[idx++] = (byte)(Math.round(pxl*255/bitNum[3]));
-                            } else {
-                                image[idx++] = (byte)pxl;
+                                pxl = Math.round(pxl*255.0f/bitNum[3]);
                             }
+                            image[idx++] = (byte)pxl;
                         }
                     }
                 }
@@ -431,10 +428,9 @@ public class ImageUtil {
                         for (int i=0; i<3; i++) {
                             int pxl = (val & bitField[i]) >>> bitShift[i];
                             if (bitNum[i] != 255) {
-                                image[idx++] = (byte)(Math.round(pxl*255/bitNum[i]));
-                            } else {
-                                image[idx++] = (byte)pxl;
+                                pxl = Math.round(pxl*255.0f/bitNum[i]);
                             }
+                            image[idx++] = (byte)pxl;
                         }
                         // alpha
                         if (bitField[3] == 0) {
@@ -443,10 +439,9 @@ public class ImageUtil {
                             val = ~val;
                             int pxl = (val & bitField[3]) >>> bitShift[3];
                             if (bitNum[3] != 255) {
-                                image[idx++] = (byte)(Math.round(pxl*255/bitNum[3]));
-                            } else {
-                                image[idx++] = (byte)pxl;
+                                pxl = Math.round(pxl*255.0f/bitNum[3]);
                             }
+                            image[idx++] = (byte)pxl;
                         }
                     }
                 }
