@@ -6,7 +6,6 @@ import java.util.Arrays;
 import java.util.Stack;
 
 import org.apache.commons.lang.CharUtils;
-import org.apache.commons.lang.StringUtils;
 
 import org.springframework.web.util.HtmlUtils;
 
@@ -191,6 +190,7 @@ public class HTMLHook extends HookAdapter<String> {
      * すべての入力をクリアし、初期化します。
      *
      */
+    @Override
     public void clear() {
         _html.delete(0, _html.length());
         _bufStack.clear();
@@ -205,6 +205,7 @@ public class HTMLHook extends HookAdapter<String> {
      *
      * @return HTML文字列
      */
+    @Override
     public String getObject() {
         if (!_stack.empty()) {
             _html.append("</dd></dl>");
@@ -222,6 +223,7 @@ public class HTMLHook extends HookAdapter<String> {
      *
      * @return まだ入力を受けつける場合はtrue、そうでない場合はfalse
      */
+    @Override
     public boolean isMoreInput() {
         if (_line >= MAX_LINE) {
             return false;
@@ -234,6 +236,7 @@ public class HTMLHook extends HookAdapter<String> {
      *
      * @param str 文字列
      */
+    @Override
     public void append(String str) {
         if (_narrow) {
             str = ByteUtil.wideToNarrow(str);
@@ -246,6 +249,7 @@ public class HTMLHook extends HookAdapter<String> {
      *
      * @param code 外字の文字コード
      */
+    @Override
     public void append(int code) {
         // 出力ファイル名
         int fore = _foreground.getRGB() & 0xffffff;
@@ -279,6 +283,7 @@ public class HTMLHook extends HookAdapter<String> {
      * 半角表示の開始を表すエスケープシーケンスに対するフックです。
      *
      */
+    @Override
     public void beginNarrow() {
         _narrow = true;
         _position = _html.length();
@@ -288,6 +293,7 @@ public class HTMLHook extends HookAdapter<String> {
      * 半角表示の終了を表すエスケープシーケンスに対するフックです。
      *
      */
+    @Override
     public void endNarrow() {
         _narrow = false;
         if (_position < _html.length()) {
@@ -299,6 +305,7 @@ public class HTMLHook extends HookAdapter<String> {
      * 下付き表示の開始を表すエスケープシーケンスに対するフックです。
      *
      */
+    @Override
     public void beginSubscript() {
         _output.append("<sub>");
     }
@@ -307,6 +314,7 @@ public class HTMLHook extends HookAdapter<String> {
      * 下付き表示の終了を表すエスケープシーケンスに対するフックです。
      *
      */
+    @Override
     public void endSubscript() {
         _output.append("</sub>");
     }
@@ -315,6 +323,7 @@ public class HTMLHook extends HookAdapter<String> {
      * 上付き表示の開始を表すエスケープシーケンスに対するフックです。
      *
      */
+    @Override
     public void beginSuperscript() {
         _output.append("<sup>");
     }
@@ -323,6 +332,7 @@ public class HTMLHook extends HookAdapter<String> {
      * 上付き表示の終了を表すエスケープシーケンスに対するフックです。
      *
      */
+    @Override
     public void endSuperscript() {
         _output.append("</sup>");
     }
@@ -332,6 +342,7 @@ public class HTMLHook extends HookAdapter<String> {
      *
      * @param indent 字下げ量
      */
+    @Override
     public void setIndent(int indent) {
         if (indent < 0) {
             return;
@@ -359,6 +370,7 @@ public class HTMLHook extends HookAdapter<String> {
      * 改行を表すエスケープシーケンスに対するフックです。
      *
      */
+    @Override
     public void newLine() {
         _output.append("<br>");
         _line++;
@@ -368,6 +380,7 @@ public class HTMLHook extends HookAdapter<String> {
      * 改行禁止の開始を表すエスケープシーケンスに対するフックです。
      *
      */
+    @Override
     public void beginNoNewLine() {
         _output.append("<span class=\"nobreak\">");
     }
@@ -376,6 +389,7 @@ public class HTMLHook extends HookAdapter<String> {
      * 改行禁止の終了を表すエスケープシーケンスに対するフックです。
      *
      */
+    @Override
     public void endNoNewLine() {
         _output.append("</span>");
     }
@@ -384,6 +398,7 @@ public class HTMLHook extends HookAdapter<String> {
      * 強調表示の開始を表すエスケープシーケンスに対するフックです。
      *
      */
+    @Override
     public void beginEmphasis() {
         _output.append("<em>");
     }
@@ -392,6 +407,7 @@ public class HTMLHook extends HookAdapter<String> {
      * 強調表示の終了を表すエスケープシーケンスに対するフックです。
      *
      */
+    @Override
     public void endEmphasis() {
         _output.append("</em>");
     }
@@ -403,6 +419,7 @@ public class HTMLHook extends HookAdapter<String> {
      * @see #BOLD
      * @see #ITALIC
      */
+    @Override
     public void beginDecoration(int type) {
         _decoration = type;
         switch (_decoration) {
@@ -422,6 +439,7 @@ public class HTMLHook extends HookAdapter<String> {
      * 文字修飾の終了を表すエスケープシーケンスに対するフックです。
      *
      */
+    @Override
     public void endDecoration() {
         switch (_decoration) {
             case 1:
@@ -440,6 +458,7 @@ public class HTMLHook extends HookAdapter<String> {
      * 複合検索の候補となる語の開始を表すエスケープシーケンスに対するフックです。
      *
      */
+    @Override
     public void beginCandidate() {
         _bufStack.push(_output);
         _output = new StringBuilder();
@@ -451,6 +470,7 @@ public class HTMLHook extends HookAdapter<String> {
      *
      * @param pos 次の階層の候補一覧データの位置
      */
+    @Override
     public void endCandidateGroup(long pos) {
         StringBuilder buf = _output;
         _output = _bufStack.pop();
@@ -466,6 +486,7 @@ public class HTMLHook extends HookAdapter<String> {
      * 候補となる語が実際に検索の入力語として使えるものであることを示します。
      *
      */
+    @Override
     public void endCandidateLeaf() {
         StringBuilder buf = _output;
         _output = _bufStack.pop();
@@ -476,6 +497,7 @@ public class HTMLHook extends HookAdapter<String> {
      * 別位置のテキストデータの参照開始を表すエスケープシーケンスに対するフックです。
      *
      */
+    @Override
     public void beginReference() {
         _foreground = _anchor;
         _bufStack.push(_output);
@@ -487,6 +509,7 @@ public class HTMLHook extends HookAdapter<String> {
      *
      * @param pos 参照先の位置
      */
+    @Override
     public void endReference(long pos) {
         StringBuilder buf = _output;
         _output = _bufStack.pop();
@@ -502,6 +525,7 @@ public class HTMLHook extends HookAdapter<String> {
      * キーワード表示の開始を表すエスケープシーケンスに対するフックです。
      *
      */
+    @Override
     public void beginKeyword() {
         _foreground = _keyword;
         _output.append("<span class=\"keyword\">");
@@ -511,6 +535,7 @@ public class HTMLHook extends HookAdapter<String> {
      * キーワード表示の終了を表すエスケープシーケンスに対するフックです。
      *
      */
+    @Override
     public void endKeyword() {
         _foreground = _default;
         _output.append("</span>");
@@ -522,6 +547,7 @@ public class HTMLHook extends HookAdapter<String> {
      * @param width 画像の幅
      * @param height 画像の高さ
      */
+    @Override
     public void beginMonoGraphic(int width, int height) {
         _width = width;
         _height = height;
@@ -534,6 +560,7 @@ public class HTMLHook extends HookAdapter<String> {
      *
      * @param pos 画像データの位置
      */
+    @Override
     public void endMonoGraphic(long pos) {
         StringBuilder buf = _output;
         _output = _bufStack.pop();
@@ -576,6 +603,7 @@ public class HTMLHook extends HookAdapter<String> {
      * @see #DIB
      * @see #JPEG
      */
+    @Override
     public void beginInlineColorGraphic(int format, long pos) {
         _imgFormat = format;
         _imgPos = pos;
@@ -587,6 +615,7 @@ public class HTMLHook extends HookAdapter<String> {
      * インラインカラー画像の参照終了を表すエスケープシーケンスに対するフックです。
      *
      */
+    @Override
     public void endInlineColorGraphic() {
         StringBuilder buf = _output;
         _output = _bufStack.pop();
@@ -612,6 +641,7 @@ public class HTMLHook extends HookAdapter<String> {
      * @param format 画像形式
      * @param pos 画像データの位置
      */
+    @Override
     public void beginColorGraphic(int format, long pos) {
         _clickable.delete(0, _clickable.length());
         _imgFormat = format;
@@ -624,6 +654,7 @@ public class HTMLHook extends HookAdapter<String> {
      * カラー画像の参照終了を表すエスケープシーケンスに対するフックです。
      *
      */
+    @Override
     public void endColorGraphic() {
         StringBuilder buf = _output;
         _output = _bufStack.pop();
@@ -670,6 +701,7 @@ public class HTMLHook extends HookAdapter<String> {
      * @param h 高さ
      * @param pos 参照先の位置
      */
+    @Override
     public void beginClickableArea(int x, int y, int w, int h, long pos) {
         if (_href != null) {
             _clickable.append("<area shape=\"rect\"");
@@ -693,6 +725,7 @@ public class HTMLHook extends HookAdapter<String> {
      * @see #WAVE
      * @see #MIDI
      */
+    @Override
     public void beginSound(int format, long start, long end) {
         _foreground = _anchor;
         _file.delete(0, _file.length());
@@ -714,6 +747,7 @@ public class HTMLHook extends HookAdapter<String> {
      * 音声の終了を表すエスケープシーケンスに対するフックです。
      *
      */
+    @Override
     public void endSound() {
         _output.append("</a>");
         if (_inlineObject) {
@@ -739,6 +773,7 @@ public class HTMLHook extends HookAdapter<String> {
      * @param height 動画の高さ
      * @param filename 動画ファイル名
      */
+    @Override
     public void beginMovie(int format, int width, int height, String filename) {
         File mpeg = _subbook.getMovieFile(filename);
         if (mpeg == null) {
@@ -765,6 +800,7 @@ public class HTMLHook extends HookAdapter<String> {
      * 動画の終了を表すエスケープシーケンスに対するフックです。
      *
      */
+    @Override
     public void endMovie() {
         if (_foreground.equals(_anchor)) {
             _output.append("</a>");
@@ -839,21 +875,33 @@ public class HTMLHook extends HookAdapter<String> {
                     }
                     idx2 = pos[index] + off2;
                     len = _html.length();
-                    int mark = 0;
+                    int dots = 0;
                     while (idx2 < len) { // 後方
                         ch = _html.charAt(idx2);
                         if (CharUtils.isAsciiAlphanumeric(ch)
-                            || ch == '+' || ch == '-' || ch == '.' || ch == '_') {
+                            || ch == '+' || ch == '-' || ch == '_') {
                             idx2++;
                             href.append(ch);
-                            if (ch == '.') {
-                                mark++;
-                            }
+                        } else if (ch == '.') {
+                            idx2++;
+                            dots++;
+                            href.append(ch);
                         } else {
                             break;
                         }
                     }
-                    if (mark > 0) {
+                    if (href.length() > 0) {
+                        ch = _html.charAt(idx2-1);
+                        if (ch == '.') {
+                            href.deleteCharAt(href.length()-1);
+                            idx2--;
+                            dots--;
+                        } else if (ch == ',') {
+                            href.deleteCharAt(href.length()-1);
+                            idx2--;
+                        }
+                    }
+                    if (dots > 0) {
                         href.insert(0, "mailto:");
                     } else {
                         idx1 = -1;
@@ -898,6 +946,13 @@ public class HTMLHook extends HookAdapter<String> {
                             break;
                         }
                     }
+                    if (href.length() > 0) {
+                        ch = _html.charAt(idx2-1);
+                        if (ch == '.' || ch == ',') {
+                            href.deleteCharAt(href.length()-1);
+                            idx2--;
+                        }
+                    }
                     break;
                 }
                 case 2: // www
@@ -925,19 +980,25 @@ public class HTMLHook extends HookAdapter<String> {
                     }
                     idx2 = pos[index] + off2;
                     len = _html.length();
+                    int dots = 1;
                     int mark = 0;
                     while (idx2 < len) { // 後方
                         ch = _html.charAt(idx2);
                         if (CharUtils.isAsciiAlphanumeric(ch)
                             || ch == '#' || ch == '%' || ch == '('
-                            || ch == '+' || ch == ',' || ch == '-' || ch == '.'
-                            || ch == '/' || ch == ':' || ch == '=' || ch == '?'
+                            || ch == '+' || ch == ',' || ch == '-'
+                            || ch == '/' || ch == '=' || ch == '?'
                             || ch == '@' || ch == '_' || ch == '~') {
                             idx2++;
                             href.append(ch);
-                            if (ch == ':') {
-                                mark++;
-                            }
+                        } else if (ch == ':') {
+                            idx2++;
+                            mark++;
+                            href.append(ch);
+                        } else if (ch == '.') {
+                            idx2++;
+                            dots++;
+                            href.append(ch);
                         } else if (ch == '&'
                                    && _html.indexOf("&amp;", idx2) == idx2) {
                             idx2 += 5;
@@ -946,9 +1007,26 @@ public class HTMLHook extends HookAdapter<String> {
                             break;
                         }
                     }
-                    if (index == 4 && mark == 0) {
-                        // urn:xxx:xxx の形式でないので無視
-                        idx1 = -1;
+                    if (href.length() > 0) {
+                        ch = _html.charAt(idx2-1);
+                        if (ch == '.') {
+                            href.deleteCharAt(href.length()-1);
+                            idx2--;
+                            dots--;
+                        } else if (ch == ',') {
+                            href.deleteCharAt(href.length()-1);
+                            idx2--;
+                        }
+                    }
+                    if (index == 4) {
+                        if (mark == 0) {
+                            // urn:xxx:xxx の形式でないので無視
+                            idx1 = -1;
+                        }
+                    } else {
+                        if (dots < 2) {
+                            idx1 = -1;
+                        }
                     }
                     break;
                 }
